'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = buildOptions;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

require('./guard');

var _hook = require('./hook');

var _hook2 = _interopRequireDefault(_hook);

var _postcss = require('postcss');

var _postcss2 = _interopRequireDefault(_postcss);

var _path = require('path');

var _fs = require('fs');

var _lodashIsplainobject = require('lodash.isplainobject');

var _lodashIsplainobject2 = _interopRequireDefault(_lodashIsplainobject);

var _postcssModulesExtractImports = require('postcss-modules-extract-imports');

var _postcssModulesExtractImports2 = _interopRequireDefault(_postcssModulesExtractImports);

var _postcssModulesLocalByDefault = require('postcss-modules-local-by-default');

var _postcssModulesLocalByDefault2 = _interopRequireDefault(_postcssModulesLocalByDefault);

var _postcssModulesScope = require('postcss-modules-scope');

var _postcssModulesScope2 = _interopRequireDefault(_postcssModulesScope);

var _parser = require('./parser');

var _parser2 = _interopRequireDefault(_parser);

var processCss = undefined;
var rootDir = undefined;
var plugins = undefined;

/**
 * @param  {object}   opts
 * @param  {function} opts.generateScopedName
 * @param  {function} opts.processCss|.p
 * @param  {string}   opts.rootDir|.root|.d
 * @param  {array}    opts.use|.u
 */

function buildOptions() {
  var opts = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  if (!(0, _lodashIsplainobject2['default'])(opts)) {
    throw new Error('Use plain object');
  }

  processCss = get(opts, 'processCss|p');
  rootDir = get(opts, 'rootDir|root|d');
  rootDir = rootDir ? (0, _path.resolve)(rootDir) : process.cwd();

  var customPlugins = get(opts, 'use|u');
  if (Array.isArray(customPlugins)) {
    return void (plugins = customPlugins);
  }

  plugins = [];

  plugins.push(opts.mode ? new _postcssModulesLocalByDefault2['default']({ mode: opts.mode }) : _postcssModulesLocalByDefault2['default']);

  plugins.push(opts.createImportedName ? new _postcssModulesExtractImports2['default']({ createImportedName: opts.createImportedName }) : _postcssModulesExtractImports2['default']);

  plugins.push(opts.generateScopedName ? new _postcssModulesScope2['default']({ generateScopedName: opts.generateScopedName }) : _postcssModulesScope2['default']);
}

var escapedSeparator = _path.sep.replace(/(.)/g, '\\$1');
var relativePathPattern = new RegExp('^.{1,2}$|^.{1,2}' + escapedSeparator);
var tokensByFile = {};
var importNr = 0;

/**
 * @param  {object} object
 * @param  {string} keys 'a|b|c'
 * @return {*}
 */
function get(object, keys) {
  var key = undefined;

  keys.split('|').some(function (k) {
    if (!object[k]) {
      return false;
    }

    key = k;
    return true;
  });

  return key ? object[key] : null;
}

/**
 * @param  {string}  pathname
 * @return {boolean}
 */
function isModule(pathname) {
  var parsed = (0, _path.parse)(pathname);
  return !parsed.root && !relativePathPattern.test(parsed.dir);
}

/**
 * @param  {string}   sourceString The file content
 * @param  {string}   sourcePath
 * @param  {string}   trace
 * @param  {function} pathFetcher
 * @return {object}
 */
function load(sourceString, sourcePath, trace, pathFetcher) {
  var lazyResult = (0, _postcss2['default'])(plugins.concat(new _parser2['default']({ pathFetcher: pathFetcher, trace: trace }))).process(sourceString, { from: sourcePath });

  return { injectableSource: lazyResult.css, exportTokens: lazyResult.root.tokens };
}

/**
 * @param  {string} _newPath
 * @param  {string} _relativeTo
 * @param  {string} _trace
 * @return {object}
 */
function fetch(_newPath, _relativeTo, _trace) {
  var newPath = _newPath.replace(/^["']|["']$/g, '');
  var trace = _trace || String.fromCharCode(importNr++);

  var relativeDir = (0, _path.dirname)(_relativeTo);
  var rootRelativePath = (0, _path.resolve)(relativeDir, newPath);
  var fileRelativePath = (0, _path.resolve)((0, _path.join)(rootDir, relativeDir), newPath);

  if (isModule(newPath)) {
    fileRelativePath = require.resolve(newPath);
  }

  var tokens = tokensByFile[fileRelativePath];
  if (tokens) {
    return tokens;
  }

  var source = (0, _fs.readFileSync)(fileRelativePath, 'utf-8');

  var _load = load(source, rootRelativePath, trace, fetch);

  var exportTokens = _load.exportTokens;
  var injectableSource = _load.injectableSource;

  tokensByFile[fileRelativePath] = exportTokens;

  if (typeof processCss === 'function') {
    processCss(injectableSource);
  }

  return exportTokens;
}

// setting defaults
buildOptions();

(0, _hook2['default'])(function (filename) {
  return fetch('.' + _path.sep + (0, _path.relative)(rootDir, filename), '/');
});
module.exports = exports['default'];